==> hardware-monitor-0.3.tar.gz: hardware-monitor-0.3/src/applet.cpp <==
/* Implementation of the Applet class.
 *
 * Copyright (c) 2003 Ole Laursen.
 *
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License as 
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA.
 */


#include <config.h>

#include <algorithm>
#include <vector>
#include <iostream>

#include <gtkmm/main.h>

#include "ucompose.hpp"

#include "applet.hpp"
#include "curve-view.hpp"
#include "bar-view.hpp"
#include "text-view.hpp"
#include "flame-view.hpp"
#include "monitor-impls.hpp"
#include "preferences-window.hpp"
#include "i18n.hpp"

// helpers for popping up the various things

void display_preferences(BonoboUIComponent *, void *applet, const gchar *)
{
  static_cast<Applet *>(applet)->on_preferences_activated();
}

void display_help(BonoboUIComponent *, void *applet, const gchar *)
{
  static_cast<Applet *>(applet)->on_help_activated();
}

void display_about(BonoboUIComponent *, void *applet, const gchar *)
{
  static_cast<Applet *>(applet)->on_about_activated();
}


Applet::Applet(PanelApplet *a)
  : panel_applet(a)
{
  // load icon
  std::string icon_name = GNOMEICONDIR "/hardware-monitor-applet.png";
  try {
    icon = Gdk::Pixbuf::create_from_file(icon_name);
  }
  catch(...) {
    std::cerr <<
      String::ucompose(_("Hardware Monitor: cannot load the icon '%1'.\n"),
		       icon_name);
    // it's a minor problem if we can't find the icon
    icon = Glib::RefPtr<Gdk::Pixbuf>();
  }
  
  // setup GConf
  //gconf_dir = "/apps/hardware-monitor";
  gconf_dir = panel_applet_get_preferences_key(panel_applet);
  gconf_client = Gnome::Conf::Client::get_default_client();
  gconf_client->add_dir(gconf_dir);

  // circumvent GConf bug (FIXME: report it)
  gconf_client->set(gconf_dir + "/dummy", 0);
  gconf_client->set(gconf_dir + "/monitors/dummy", 0);
  
  // connect GConf
  gconf_client->notify_add(gconf_dir + "/viewer_type",
			   SigC::slot(*this, &Applet::viewer_type_listener));

  gconf_client->notify_add(gconf_dir + "/update_interval",
			   SigC::slot(*this, &Applet::update_interval_listener));

  load_monitors();
  
  preferences_window.reset(new PreferencesWindow(*this));

  set_shadow_type(Gtk::SHADOW_ETCHED_IN);

  // we need this #if 0 so the string will be in the message catalog for
  // translation below in menu_xml
#if 0
  // pop-up menu strings
    _("_Preferences...") _("_Help") _("_About...")
#endif
  
  // setup menu
  static const char menu_xml[] =
    "<popup name=\"button3\">\n"
    "   <menuitem name=\"Properties Item\""
    "             verb=\"HardwareMonitorPreferences\" _label=\"_Preferences...\"\n"
    "             pixtype=\"stock\" pixname=\"gtk-properties\"/>\n"
    "   <menuitem name=\"Help Item\""
    "             verb=\"HardwareMonitorHelp\" _label=\"_Help\"\n"
    "             pixtype=\"stock\" pixname=\"gtk-help\"/>\n"
    "   <menuitem name=\"About Item\""
    "             verb=\"HardwareMonitorAbout\" _label=\"_About...\"\n"
    "             pixtype=\"stock\" pixname=\"gnome-stock-about\"/>\n"
    "</popup>\n";

  static const BonoboUIVerb menu_verbs[] = {
    BONOBO_UI_VERB("HardwareMonitorPreferences", &display_preferences),
    BONOBO_UI_VERB("HardwareMonitorHelp", &display_help),
    BONOBO_UI_VERB("HardwareMonitorAbout", &display_about),
    
    BONOBO_UI_VERB_END
  };
  
  panel_applet_setup_menu(panel_applet, menu_xml, menu_verbs, this);
  
  // start displaying something
  viewer_type_listener(0, gconf_client->get_entry(gconf_dir + "/viewer_type"));
  update_interval_listener(0, gconf_client->get_entry(gconf_dir
						      + "/update_interval"));
}

Applet::~Applet()
{
  // make sure noone is trying to read the monitors before we kill them
  view.reset();
  
  for (monitor_iterator i = monitors.begin(), end = monitors.end(); i != end;
       ++i)
    delete i->second;
}

void Applet::set_view(View *v)
{
  view.reset(v);
  view->display(*this);

  for (monitor_iterator i = monitors.begin(), end = monitors.end(); i != end;
       ++i)
    view->attach(i->second, i->first);
}

bool Applet::main_loop()
{
  if (view.get())
    view->update();
  
  return true;
}

Gtk::Container &Applet::get_container()
{
  // FIXME: return frame
  return *this;
}

unsigned int Applet::get_fg_color()
{
  static unsigned int colors[] = {
    0x83A67F, 0xC1665A, 0x7590AE, 0xE0C39E, 0x887FA3
  };

  Gnome::Conf::Value v = gconf_client->get(gconf_dir + "/next_color");
  int c;
      
  if (v.get_type() == Gnome::Conf::VALUE_INT)
    c = v.get_int();
  else
    c = 0;

 gconf_client->set(gconf_dir + "/next_color",
		   int((c + 1) % (sizeof(colors) / sizeof(unsigned int))));
  
 return colors[c];
}

int Applet::get_size() const
{
  // 2 pixels of frame decoration on both sides
  return panel_applet_get_size(panel_applet) - 2 * 2;
}

bool Applet::horizontal() const
{
  PanelAppletOrient orient = panel_applet_get_orient(panel_applet);
  return orient == PANEL_APPLET_ORIENT_UP || orient == PANEL_APPLET_ORIENT_DOWN;
}

Glib::RefPtr<Gnome::Conf::Client> &Applet::get_gconf_client()
{
  return gconf_client;
}

Glib::ustring Applet::get_gconf_dir() const
{
  return gconf_dir;
}

Glib::RefPtr<Gdk::Pixbuf> Applet::get_icon()
{
  return icon;
}

void Applet::add_monitor(Monitor *monitor)
{
  Glib::ustring mon_dir = find_empty_monitor_dir();

  add_monitor(monitor, mon_dir);
}

void Applet::remove_monitor(Glib::ustring mon_dir)
{
  Monitor *monitor = monitors[mon_dir];
  monitors.erase(mon_dir);
  
  if (view.get())
    view->detach(monitor);

  delete monitor;
  
  // delete the GConf keys
  Glib::SListHandle<Gnome::Conf::Entry> list
    = gconf_client->all_entries(mon_dir);

  for (Glib::SListHandle<Gnome::Conf::Entry>::const_iterator i = list.begin(),
	end = list.end(); i != end; ++i)
    gconf_client->unset((*i).get_key());
}

void Applet::replace_monitor(Glib::ustring mon_dir, Monitor *new_mon)
{
  Monitor *old_mon = monitors[mon_dir];
  monitors[mon_dir] = new_mon;

  new_mon->save(gconf_client, mon_dir);
  
  if (view.get()) {
    view->detach(old_mon);
    view->attach(new_mon, mon_dir);
  }

  delete old_mon;
}

Monitor *Applet::look_up_monitor(Glib::ustring mon_dir)
{
  return monitors[mon_dir];
}

void Applet::viewer_type_listener(unsigned int, Gnome::Conf::Entry entry)
{
  if (entry.get_value().get_type() != Gnome::Conf::VALUE_STRING) {
    // FIXME: use schema for this?
    Gnome::Conf::Value v(Gnome::Conf::VALUE_STRING);
    v.set(Glib::ustring("curve"));
    entry.set_value(v);

    gconf_client->set(entry.get_key(), Glib::ustring("curve"));
  }

  Glib::ustring s = entry.get_value().get_string();
  
  if (s == "curve") {
    if (!dynamic_cast<CurveView *>(view.get()))
      set_view(new CurveView);
  }
  else if (s == "bar") {
    if (!dynamic_cast<BarView *>(view.get()))
      set_view(new BarView);
  }
  else if (s == "text") {
    if (!dynamic_cast<TextView *>(view.get()))
      set_view(new TextView);
  }
  else if (s == "flame") {
    if (!dynamic_cast<FlameView *>(view.get()))
      set_view(new FlameView);
  }
}

void Applet::update_interval_listener(unsigned int, Gnome::Conf::Entry entry)
{
  if (entry.get_value().get_type() != Gnome::Conf::VALUE_FLOAT) {
    // FIXME: use schema for this?
    Gnome::Conf::Value v(Gnome::Conf::VALUE_FLOAT);
    v.set(1.0);
    entry.set_value(v);
  }
  
  update_interval = int(1000 * entry.get_value().get_float());
  
  timer.disconnect();
  timer =
    Glib::signal_timeout().connect(SigC::slot(*this, &Applet::main_loop),
				   update_interval);
}

Glib::ustring Applet::find_empty_monitor_dir()
{
  Glib::SListHandle<Glib::ustring> list
    = gconf_client->all_dirs(gconf_dir + "/monitors");

  Glib::SListHandle<Glib::ustring>::const_iterator
    begin = list.begin(),
    end = list.end();

  int c = 1;
  Glib::ustring mon_dir = String::ucompose(gconf_dir + "/monitors/%1", c);
  while (std::find(begin, end, mon_dir) != end)
    mon_dir = String::ucompose(gconf_dir + "/monitors/%1", ++c);

  return mon_dir;
}

void Applet::add_monitor(Monitor *monitor, Glib::ustring mon_dir)
{
  monitors[mon_dir] = monitor;
  monitor->save(gconf_client, mon_dir);
  
  if (view.get())
    view->attach(monitor, mon_dir);
}

void Applet::load_monitors()
{
  Glib::SListHandle<Glib::ustring> list
    = gconf_client->all_dirs(gconf_dir + "/monitors");

  for (Glib::SListHandle<Glib::ustring>::const_iterator i = list.begin(),
	 end = list.end(); i != end; ++i) {
    const Glib::ustring &mon_dir = *i;
    
    Glib::ustring type = gconf_client->get_string(mon_dir + "/type");
    
    if (type == "cpu_usage") {
      Gnome::Conf::Value v = gconf_client->get(mon_dir + "/cpu_no");
      int cpu_no;
      
      if (v.get_type() == Gnome::Conf::VALUE_INT)
	cpu_no = v.get_int();
      else
	cpu_no = -1;

      if (cpu_no == -1)
	add_monitor(new CpuUsageMonitor, mon_dir);
      else
	add_monitor(new CpuUsageMonitor(cpu_no), mon_dir);
    }
    else if (type == "memory_usage")
      add_monitor(new MemoryUsageMonitor, mon_dir);
    else if (type == "swap_usage")
      add_monitor(new SwapUsageMonitor, mon_dir);
    else if (type == "load_average")
      add_monitor(new LoadAverageMonitor, mon_dir);
    else if (type == "disk_usage") {
      Gnome::Conf::Value v = gconf_client->get(mon_dir + "/mount_dir");
      Glib::ustring mount_dir;
      
      if (v.get_type() == Gnome::Conf::VALUE_STRING)
	mount_dir = v.get_string();
      else
	mount_dir = "/";	// FIXME: use schema?
      
      add_monitor(new DiskUsageMonitor(mount_dir), mon_dir);
    }
    else if (type == "network_load") {
      Glib::ustring inter;
      int inter_no, inter_direction;
      
      Gnome::Conf::Value v = gconf_client->get(mon_dir + "/interface");
      
      if (v.get_type() == Gnome::Conf::VALUE_STRING)
	inter = v.get_string();
      else
	inter = "eth";	// FIXME: use schema?

      
      v = gconf_client->get(mon_dir + "/interface_no");
      
      if (v.get_type() == Gnome::Conf::VALUE_INT)
	inter_no = v.get_int();
      else
	inter_no = 0;	// FIXME: use schema?

      
      v = gconf_client->get(mon_dir + "/interface_direction");
      
      if (v.get_type() == Gnome::Conf::VALUE_INT)
	inter_direction = v.get_int();
      else
	inter_direction = NetworkLoadMonitor::all_data; // FIXME: use schema?

      NetworkLoadMonitor::Direction dir;
      
      if (inter_direction == NetworkLoadMonitor::incoming_data)
	dir = NetworkLoadMonitor::incoming_data;
      else if (inter_direction == NetworkLoadMonitor::outgoing_data)
	dir = NetworkLoadMonitor::outgoing_data;
      else
	dir = NetworkLoadMonitor::all_data;

      add_monitor(new NetworkLoadMonitor(inter, inter_no, dir), mon_dir);
    }
    else if (type == "temperature")
      add_monitor(new TemperatureMonitor, mon_dir);
  }

  // always start with a CpuUsageMonitor - FIXME: use schema?
  if (monitors.empty())
    add_monitor(new CpuUsageMonitor);
}

void Applet::on_preferences_activated()
{
  preferences_window->show();
}

void Applet::on_help_activated()
{
  // FIXME: do something
}

void Applet::on_about_activated()
{
  std::vector<Glib::ustring> authors;
  authors.push_back("Ole Laursen <olau@hardworking.dk>");
  
  std::vector<Glib::ustring> documenters;
  // add documenters here

  // note to translators: please fill in your names and email addresses
  Glib::ustring translators = _("translator_credits");

  Glib::ustring description =
    _("Monitor various hardware-related information, such as CPU usage, "
      "memory usage etc. Supports curve graphs, horizontal bar plots, "
      "textual monitoring and fluctuating flames.");
  
  if (about.get() == 0) {
    about.reset(new Gnome::UI::About(_("Hardware Monitor"), VERSION,
				     _("Copyright (c) 2003 Ole Laursen"),
				     authors, documenters, description,
				     translators == "translator_credits"
				     ? "" : translators,
				     icon));
    about->set_icon(icon);
    about->show();
  }
  else {
    about->show();
    about->raise();
  }
}

==> hardware-monitor-0.4.tar.gz: hardware-monitor-0.4/src/applet.cpp <==
/* Implementation of the Applet class.
 *
 * Copyright (c) 2003 Ole Laursen.
 *
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License as 
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA.
 */


#include <config.h>

#include <algorithm>
#include <vector>
#include <iostream>

#include <gtkmm/main.h>

#include "ucompose.hpp"

#include "applet.hpp"
#include "curve-view.hpp"
#include "bar-view.hpp"
#include "text-view.hpp"
#include "flame-view.hpp"
#include "monitor-impls.hpp"
#include "preferences-window.hpp"
#include "i18n.hpp"

// helpers for popping up the various things

void display_preferences(BonoboUIComponent *, void *applet, const gchar *)
{
  static_cast<Applet *>(applet)->on_preferences_activated();
}

void display_help(BonoboUIComponent *, void *applet, const gchar *)
{
  static_cast<Applet *>(applet)->on_help_activated();
}

void display_about(BonoboUIComponent *, void *applet, const gchar *)
{
  static_cast<Applet *>(applet)->on_about_activated();
}


Applet::Applet(PanelApplet *a)
  : panel_applet(a)
{
  // load icon
  std::string icon_name = GNOMEICONDIR "/hardware-monitor-applet.png";
  try {
    icon = Gdk::Pixbuf::create_from_file(icon_name);
  }
  catch(...) {
    std::cerr <<
      String::ucompose(_("Hardware Monitor: cannot load the icon '%1'.\n"),
		       icon_name);
    // it's a minor problem if we can't find the icon
    icon = Glib::RefPtr<Gdk::Pixbuf>();
  }
  
  // setup GConf
  //gconf_dir = "/apps/hardware-monitor";
  gconf_dir = panel_applet_get_preferences_key(panel_applet);
  gconf_client = Gnome::Conf::Client::get_default_client();
  gconf_client->add_dir(gconf_dir);

  // circumvent GConf bug (FIXME: report it)
  gconf_client->set(gconf_dir + "/dummy", 0);
  gconf_client->set(gconf_dir + "/monitors/dummy", 0);
  
  // connect GConf
  gconf_client->notify_add(gconf_dir + "/viewer_type",
			   SigC::slot(*this, &Applet::viewer_type_listener));

  gconf_client->notify_add(gconf_dir + "/update_interval",
			   SigC::slot(*this, &Applet::
				      update_interval_listener));
  gconf_client->notify_add(gconf_dir + "/background_color",
			   SigC::slot(*this, &Applet::
				      background_color_listener));
  gconf_client->notify_add(gconf_dir + "/use_background_color",
			   SigC::slot(*this, &Applet::
				      use_background_color_listener));

  load_monitors();
  
  preferences_window.reset(new PreferencesWindow(*this));

  set_shadow_type(Gtk::SHADOW_ETCHED_IN);

  // we need this #if 0 so the string will be in the message catalog for
  // translation below in menu_xml
#if 0
  // pop-up menu strings
    _("_Preferences...") _("_Help") _("_About...")
#endif
  
  // setup menu
  static const char menu_xml[] =
    "<popup name=\"button3\">\n"
    "   <menuitem name=\"Properties Item\"\n"
    "             verb=\"HardwareMonitorPreferences\" _label=\"_Preferences...\"\n"
    "             pixtype=\"stock\" pixname=\"gtk-properties\"/>\n"
    "   <menuitem name=\"Help Item\"\n"
    "             verb=\"HardwareMonitorHelp\" _label=\"_Help\"\n"
    "             pixtype=\"stock\" pixname=\"gtk-help\"/>\n"
    "   <menuitem name=\"About Item\"\n"
    "             verb=\"HardwareMonitorAbout\" _label=\"_About...\"\n"
    "             pixtype=\"stock\" pixname=\"gnome-stock-about\"/>\n"
    "</popup>\n";

  static const BonoboUIVerb menu_verbs[] = {
    BONOBO_UI_VERB("HardwareMonitorPreferences", &display_preferences),
    BONOBO_UI_VERB("HardwareMonitorHelp", &display_help),
    BONOBO_UI_VERB("HardwareMonitorAbout", &display_about),
    
    BONOBO_UI_VERB_END
  };
  
  panel_applet_setup_menu(panel_applet, menu_xml, menu_verbs, this);
  
  // start displaying something
  viewer_type_listener(0, gconf_client->get_entry(gconf_dir + "/viewer_type"));
  update_interval_listener(0, gconf_client->get_entry(gconf_dir
						      + "/update_interval"));
  background_color_listener(0, gconf_client->get_entry(gconf_dir
						       + "/background_color"));
  use_background_color_listener(0, gconf_client->get_entry(gconf_dir
							   + "/use_background_color"));

  view->update();
}

Applet::~Applet()
{
  // make sure noone is trying to read the monitors before we kill them
  view.reset();
  
  for (monitor_iterator i = monitors.begin(), end = monitors.end(); i != end;
       ++i)
    delete i->second;
}

void Applet::set_view(View *v)
{
  view.reset(v);
  view->display(*this);

  for (monitor_iterator i = monitors.begin(), end = monitors.end(); i != end;
       ++i)
    view->attach(i->second, i->first);
}

bool Applet::main_loop()
{
  if (view.get())
    view->update();
  
  return true;
}

Gtk::Container &Applet::get_container()
{
  // FIXME: return frame
  return *this;
}

unsigned int Applet::get_fg_color()
{
  static unsigned int colors[] = {
    0x83A67F, 0xC1665A, 0x7590AE, 0xE0C39E, 0x887FA3
  };

  Gnome::Conf::Value v = gconf_client->get(gconf_dir + "/next_color");
  int c;
      
  if (v.get_type() == Gnome::Conf::VALUE_INT)
    c = v.get_int();
  else
    c = 0;

 gconf_client->set(gconf_dir + "/next_color",
		   int((c + 1) % (sizeof(colors) / sizeof(unsigned int))));
  
 return colors[c];
}

int Applet::get_size() const
{
  // 2 pixels of frame decoration on both sides
  return panel_applet_get_size(panel_applet) - 2 * 2;
}

bool Applet::horizontal() const
{
  PanelAppletOrient orient = panel_applet_get_orient(panel_applet);
  return orient == PANEL_APPLET_ORIENT_UP || orient == PANEL_APPLET_ORIENT_DOWN;
}

Glib::RefPtr<Gnome::Conf::Client> &Applet::get_gconf_client()
{
  return gconf_client;
}

Glib::ustring Applet::get_gconf_dir() const
{
  return gconf_dir;
}

Glib::RefPtr<Gdk::Pixbuf> Applet::get_icon()
{
  return icon;
}

void Applet::add_monitor(Monitor *monitor)
{
  Glib::ustring mon_dir = find_empty_monitor_dir();

  add_monitor(monitor, mon_dir);
}

void Applet::remove_monitor(Glib::ustring mon_dir)
{
  Monitor *monitor = monitors[mon_dir];
  monitors.erase(mon_dir);
  
  if (view.get())
    view->detach(monitor);

  delete monitor;
  
  // delete the GConf keys
  Glib::SListHandle<Gnome::Conf::Entry> list
    = gconf_client->all_entries(mon_dir);

  for (Glib::SListHandle<Gnome::Conf::Entry>::const_iterator i = list.begin(),
	end = list.end(); i != end; ++i)
    gconf_client->unset((*i).get_key());
}

void Applet::replace_monitor(Glib::ustring mon_dir, Monitor *new_mon)
{
  Monitor *old_mon = monitors[mon_dir];
  monitors[mon_dir] = new_mon;

  new_mon->save(gconf_client, mon_dir);
  
  if (view.get()) {
    view->detach(old_mon);
    view->attach(new_mon, mon_dir);
  }

  delete old_mon;
}

Monitor *Applet::look_up_monitor(Glib::ustring mon_dir)
{
  return monitors[mon_dir];
}

void Applet::viewer_type_listener(unsigned int, Gnome::Conf::Entry entry)
{
  if (entry.get_value().get_type() != Gnome::Conf::VALUE_STRING) {
    // FIXME: use schema for this?
    Gnome::Conf::Value v(Gnome::Conf::VALUE_STRING);
    v.set(Glib::ustring("curve"));
    entry.set_value(v);

    gconf_client->set(entry.get_key(), Glib::ustring("curve"));
  }

  Glib::ustring s = entry.get_value().get_string();
  
  if (s == "curve") {
    if (!dynamic_cast<CurveView *>(view.get()))
      set_view(new CurveView);
  }
  else if (s == "bar") {
    if (!dynamic_cast<BarView *>(view.get()))
      set_view(new BarView);
  }
  else if (s == "text") {
    if (!dynamic_cast<TextView *>(view.get()))
      set_view(new TextView);
  }
  else if (s == "flame") {
    if (!dynamic_cast<FlameView *>(view.get()))
      set_view(new FlameView);
  }

  // make sure the view sets the background
  background_color_listener(0, gconf_client->get_entry(gconf_dir
						       + "/background_color"));
}

void Applet::update_interval_listener(unsigned int, Gnome::Conf::Entry entry)
{
  if (entry.get_value().get_type() != Gnome::Conf::VALUE_FLOAT) {
    // FIXME: use schema for this?
    Gnome::Conf::Value v(Gnome::Conf::VALUE_FLOAT);
    v.set(1.0);
    entry.set_value(v);
  }
  
  update_interval = int(1000 * entry.get_value().get_float());
  
  timer.disconnect();
  timer =
    Glib::signal_timeout().connect(SigC::slot(*this, &Applet::main_loop),
				   update_interval);
}

void Applet::background_color_listener(unsigned int, Gnome::Conf::Entry entry)
{
  if (entry.get_value().get_type() != Gnome::Conf::VALUE_INT) {
    // FIXME: use schema for this?
    Gnome::Conf::Value v(Gnome::Conf::VALUE_INT);
    v.set(0x00000000);		// black as the night
    entry.set_value(v);
  }

  unsigned int c = entry.get_value().get_int();

  Gnome::Conf::Value v
    = gconf_client->get(gconf_dir + "/use_background_color");

  bool use;

  if (v.get_type() != Gnome::Conf::VALUE_BOOL)
    use = false;
  else
    use = v.get_bool();
  
  view->set_background(c, use);
}

void Applet::use_background_color_listener(unsigned int, Gnome::Conf::Entry entry)
{
  if (entry.get_value().get_type() != Gnome::Conf::VALUE_BOOL) {
    // FIXME: use schema for this?
    Gnome::Conf::Value v(Gnome::Conf::VALUE_BOOL);
    v.set(false);
    entry.set_value(v);
  }

  bool use = entry.get_value().get_bool();

  Gnome::Conf::Value v
    = gconf_client->get(gconf_dir + "/background_color");

  unsigned int c;

  if (v.get_type() != Gnome::Conf::VALUE_INT)
    c = 0x00000000;
  else
    c = v.get_int();
  
  view->set_background(c, use);
}

Glib::ustring Applet::find_empty_monitor_dir()
{
  Glib::SListHandle<Glib::ustring> list
    = gconf_client->all_dirs(gconf_dir + "/monitors");

  Glib::SListHandle<Glib::ustring>::const_iterator
    begin = list.begin(),
    end = list.end();

  int c = 1;
  Glib::ustring mon_dir = String::ucompose(gconf_dir + "/monitors/%1", c);
  while (std::find(begin, end, mon_dir) != end)
    mon_dir = String::ucompose(gconf_dir + "/monitors/%1", ++c);

  return mon_dir;
}

void Applet::add_monitor(Monitor *monitor, Glib::ustring mon_dir)
{
  monitors[mon_dir] = monitor;
  monitor->save(gconf_client, mon_dir);
  
  if (view.get())
    view->attach(monitor, mon_dir);
}

void Applet::load_monitors()
{
  Glib::SListHandle<Glib::ustring> list
    = gconf_client->all_dirs(gconf_dir + "/monitors");

  for (Glib::SListHandle<Glib::ustring>::const_iterator i = list.begin(),
	 end = list.end(); i != end; ++i) {
    const Glib::ustring &mon_dir = *i;
    
    Glib::ustring type = gconf_client->get_string(mon_dir + "/type");
    
    if (type == "cpu_usage") {
      Gnome::Conf::Value v = gconf_client->get(mon_dir + "/cpu_no");
      int cpu_no;
      
      if (v.get_type() == Gnome::Conf::VALUE_INT)
	cpu_no = v.get_int();
      else
	cpu_no = -1;

      if (cpu_no == -1)
	add_monitor(new CpuUsageMonitor, mon_dir);
      else
	add_monitor(new CpuUsageMonitor(cpu_no), mon_dir);
    }
    else if (type == "memory_usage")
      add_monitor(new MemoryUsageMonitor, mon_dir);
    else if (type == "swap_usage")
      add_monitor(new SwapUsageMonitor, mon_dir);
    else if (type == "load_average")
      add_monitor(new LoadAverageMonitor, mon_dir);
    else if (type == "disk_usage") {
      Gnome::Conf::Value v = gconf_client->get(mon_dir + "/mount_dir");
      Glib::ustring mount_dir;
      
      if (v.get_type() == Gnome::Conf::VALUE_STRING)
	mount_dir = v.get_string();
      else
	mount_dir = "/";	// FIXME: use schema?
      
      add_monitor(new DiskUsageMonitor(mount_dir), mon_dir);
    }
    else if (type == "network_load") {
      Glib::ustring inter;
      int inter_no, inter_direction;
      
      Gnome::Conf::Value v = gconf_client->get(mon_dir + "/interface");
      
      if (v.get_type() == Gnome::Conf::VALUE_STRING)
	inter = v.get_string();
      else
	inter = "eth";	// FIXME: use schema?

      
      v = gconf_client->get(mon_dir + "/interface_no");
      
      if (v.get_type() == Gnome::Conf::VALUE_INT)
	inter_no = v.get_int();
      else
	inter_no = 0;	// FIXME: use schema?

      
      v = gconf_client->get(mon_dir + "/interface_direction");
      
      if (v.get_type() == Gnome::Conf::VALUE_INT)
	inter_direction = v.get_int();
      else
	inter_direction = NetworkLoadMonitor::all_data; // FIXME: use schema?

      NetworkLoadMonitor::Direction dir;
      
      if (inter_direction == NetworkLoadMonitor::incoming_data)
	dir = NetworkLoadMonitor::incoming_data;
      else if (inter_direction == NetworkLoadMonitor::outgoing_data)
	dir = NetworkLoadMonitor::outgoing_data;
      else
	dir = NetworkLoadMonitor::all_data;

      add_monitor(new NetworkLoadMonitor(inter, inter_no, dir), mon_dir);
    }
    else if (type == "temperature")
      add_monitor(new TemperatureMonitor, mon_dir);
  }

  // always start with a CpuUsageMonitor - FIXME: use schema?
  if (monitors.empty())
    add_monitor(new CpuUsageMonitor);
}

void Applet::on_preferences_activated()
{
  preferences_window->show();
}

void Applet::on_help_activated()
{
  // FIXME: do something
}

void Applet::on_about_activated()
{
  std::vector<Glib::ustring> authors;
  authors.push_back("Ole Laursen <olau@hardworking.dk>");
  
  std::vector<Glib::ustring> documenters;
  // add documenters here

  // note to translators: please fill in your names and email addresses
  Glib::ustring translators = _("translator_credits");

  Glib::ustring description =
    _("Monitor various hardware-related information, such as CPU usage, "
      "memory usage etc. Supports curve graphs, horizontal bar plots, "
      "textual monitoring and fluctuating flames.");
  
  if (about.get() == 0) {
    about.reset(new Gnome::UI::About(_("Hardware Monitor"), VERSION,
				     _("Copyright (c) 2003 Ole Laursen"),
				     authors, documenters, description,
				     translators == "translator_credits"
				     ? "" : translators,
				     icon));
    about->set_icon(icon);
    about->show();
  }
  else {
    about->show();
    about->raise();
  }
}
